"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = exports.getConnectionOptions = exports.linkRepository = exports.transaction = exports.getConnection = exports.connection = exports.collections = exports.isInitialized = void 0;
const typeorm_1 = require("typeorm");
const config_1 = __importDefault(require("./config"));
const entities_1 = require("./databases/entities");
const config_2 = require("./databases/config");
exports.isInitialized = false;
exports.collections = {};
const getConnection = () => exports.connection;
exports.getConnection = getConnection;
async function transaction(fn) {
    return exports.connection.transaction(fn);
}
exports.transaction = transaction;
function linkRepository(entityClass) {
    return exports.connection.getRepository(entityClass);
}
exports.linkRepository = linkRepository;
function getConnectionOptions(dbType) {
    switch (dbType) {
        case 'postgresdb':
            const sslCa = config_1.default.getEnv('database.postgresdb.ssl.ca');
            const sslCert = config_1.default.getEnv('database.postgresdb.ssl.cert');
            const sslKey = config_1.default.getEnv('database.postgresdb.ssl.key');
            const sslRejectUnauthorized = config_1.default.getEnv('database.postgresdb.ssl.rejectUnauthorized');
            let ssl;
            if (sslCa !== '' || sslCert !== '' || sslKey !== '' || !sslRejectUnauthorized) {
                ssl = {
                    ca: sslCa || undefined,
                    cert: sslCert || undefined,
                    key: sslKey || undefined,
                    rejectUnauthorized: sslRejectUnauthorized,
                };
            }
            return {
                ...(0, config_2.getPostgresConnectionOptions)(),
                ...(0, config_2.getOptionOverrides)('postgresdb'),
                ssl,
            };
        case 'mariadb':
        case 'mysqldb':
            return {
                ...(dbType === 'mysqldb' ? (0, config_2.getMysqlConnectionOptions)() : (0, config_2.getMariaDBConnectionOptions)()),
                ...(0, config_2.getOptionOverrides)('mysqldb'),
                timezone: 'Z',
            };
        case 'sqlite':
            return (0, config_2.getSqliteConnectionOptions)();
        default:
            throw new Error(`The database "${dbType}" is currently not supported!`);
    }
}
exports.getConnectionOptions = getConnectionOptions;
async function init(testConnectionOptions) {
    if (exports.isInitialized)
        return exports.collections;
    const dbType = config_1.default.getEnv('database.type');
    const connectionOptions = testConnectionOptions !== null && testConnectionOptions !== void 0 ? testConnectionOptions : getConnectionOptions(dbType);
    let loggingOption = config_1.default.getEnv('database.logging.enabled');
    if (loggingOption) {
        const optionsString = config_1.default.getEnv('database.logging.options').replace(/\s+/g, '');
        if (optionsString === 'all') {
            loggingOption = optionsString;
        }
        else {
            loggingOption = optionsString.split(',');
        }
    }
    const maxQueryExecutionTime = config_1.default.getEnv('database.logging.maxQueryExecutionTime');
    Object.assign(connectionOptions, {
        entities: Object.values(entities_1.entities),
        synchronize: false,
        logging: loggingOption,
        maxQueryExecutionTime,
        migrationsTransactionMode: 'each',
    });
    exports.connection = new typeorm_1.DataSource(connectionOptions);
    await exports.connection.initialize();
    if (dbType === 'postgresdb') {
        const schema = config_1.default.getEnv('database.postgresdb.schema');
        const searchPath = ['public'];
        if (schema !== 'public') {
            await exports.connection.query(`CREATE SCHEMA IF NOT EXISTS ${schema}`);
            searchPath.unshift(schema);
        }
        await exports.connection.query(`SET search_path TO ${searchPath.join(',')};`);
    }
    if (!testConnectionOptions && dbType === 'sqlite') {
        let migrations = [];
        try {
            const entityPrefix = config_1.default.getEnv('database.tablePrefix');
            migrations = await exports.connection.query(`SELECT id FROM ${entityPrefix}migrations where name = "MakeStoppedAtNullable1607431743769"`);
        }
        catch (error) {
        }
        await exports.connection.runMigrations({ transaction: 'each' });
        if (migrations.length === 0) {
            await exports.connection.destroy();
            exports.connection = new typeorm_1.DataSource(connectionOptions);
            await exports.connection.initialize();
        }
    }
    else {
        await exports.connection.runMigrations({ transaction: 'each' });
    }
    exports.collections.Credentials = linkRepository(entities_1.entities.CredentialsEntity);
    exports.collections.Execution = linkRepository(entities_1.entities.ExecutionEntity);
    exports.collections.Workflow = linkRepository(entities_1.entities.WorkflowEntity);
    exports.collections.Webhook = linkRepository(entities_1.entities.WebhookEntity);
    exports.collections.Tag = linkRepository(entities_1.entities.TagEntity);
    exports.collections.Role = linkRepository(entities_1.entities.Role);
    exports.collections.User = linkRepository(entities_1.entities.User);
    exports.collections.AuthIdentity = linkRepository(entities_1.entities.AuthIdentity);
    exports.collections.AuthProviderSyncHistory = linkRepository(entities_1.entities.AuthProviderSyncHistory);
    exports.collections.SharedCredentials = linkRepository(entities_1.entities.SharedCredentials);
    exports.collections.SharedWorkflow = linkRepository(entities_1.entities.SharedWorkflow);
    exports.collections.Settings = linkRepository(entities_1.entities.Settings);
    exports.collections.InstalledPackages = linkRepository(entities_1.entities.InstalledPackages);
    exports.collections.InstalledNodes = linkRepository(entities_1.entities.InstalledNodes);
    exports.collections.WorkflowStatistics = linkRepository(entities_1.entities.WorkflowStatistics);
    exports.collections.EventDestinations = linkRepository(entities_1.entities.EventDestinations);
    exports.isInitialized = true;
    return exports.collections;
}
exports.init = init;
//# sourceMappingURL=Db.js.map