import type express from 'express';
import type { IConnections, ICredentialDataDecryptedObject, ICredentialNodeAccess, INode, INodeCredentialTestRequest, IPinData, IRunData, IWorkflowSettings } from 'n8n-workflow';
import type { PublicUser, IExecutionDeleteFilter, IWorkflowDb } from './Interfaces';
import type { Role } from './databases/entities/Role';
import type { User } from './databases/entities/User';
import type * as UserManagementMailer from './UserManagement/email/UserManagementMailer';
export declare class UserUpdatePayload implements Pick<User, 'email' | 'firstName' | 'lastName'> {
    email: string;
    firstName: string;
    lastName: string;
}
export type AuthlessRequest<RouteParams = {}, ResponseBody = {}, RequestBody = {}, RequestQuery = {}> = express.Request<RouteParams, ResponseBody, RequestBody, RequestQuery>;
export type AuthenticatedRequest<RouteParams = {}, ResponseBody = {}, RequestBody = {}, RequestQuery = {}> = Omit<express.Request<RouteParams, ResponseBody, RequestBody, RequestQuery>, 'user'> & {
    user: User;
    mailer?: UserManagementMailer.UserManagementMailer;
    globalMemberRole?: Role;
};
export declare namespace WorkflowRequest {
    type CreateUpdatePayload = Partial<{
        id: string;
        name: string;
        nodes: INode[];
        connections: IConnections;
        settings: IWorkflowSettings;
        active: boolean;
        tags: string[];
        hash: string;
    }>;
    type ManualRunPayload = {
        workflowData: IWorkflowDb;
        runData: IRunData;
        pinData: IPinData;
        startNodes?: string[];
        destinationNode?: string;
    };
    type Create = AuthenticatedRequest<{}, {}, CreateUpdatePayload>;
    type Get = AuthenticatedRequest<{
        id: string;
    }>;
    type Delete = Get;
    type Update = AuthenticatedRequest<{
        id: string;
    }, {}, CreateUpdatePayload, {
        forceSave?: string;
    }>;
    type NewName = AuthenticatedRequest<{}, {}, {}, {
        name?: string;
    }>;
    type GetAll = AuthenticatedRequest<{}, {}, {}, {
        filter: string;
    }>;
    type GetAllActive = AuthenticatedRequest;
    type GetAllActivationErrors = Get;
    type ManualRun = AuthenticatedRequest<{}, {}, ManualRunPayload>;
    type Share = AuthenticatedRequest<{
        workflowId: string;
    }, {}, {
        shareWithIds: string[];
    }>;
}
export declare namespace CredentialRequest {
    type CredentialProperties = Partial<{
        id: string;
        name: string;
        type: string;
        nodesAccess: ICredentialNodeAccess[];
        data: ICredentialDataDecryptedObject;
    }>;
    type Create = AuthenticatedRequest<{}, {}, CredentialProperties>;
    type Get = AuthenticatedRequest<{
        id: string;
    }, {}, {}, Record<string, string>>;
    type Delete = Get;
    type GetAll = AuthenticatedRequest<{}, {}, {}, {
        filter: string;
    }>;
    type Update = AuthenticatedRequest<{
        id: string;
    }, {}, CredentialProperties>;
    type NewName = WorkflowRequest.NewName;
    type Test = AuthenticatedRequest<{}, {}, INodeCredentialTestRequest>;
    type Share = AuthenticatedRequest<{
        credentialId: string;
    }, {}, {
        shareWithIds: string[];
    }>;
}
export declare namespace ExecutionRequest {
    namespace QueryParam {
        type GetAll = {
            filter: string;
            limit: string;
            lastId: string;
            firstId: string;
        };
        type GetAllCurrent = {
            filter: string;
        };
    }
    type GetAll = AuthenticatedRequest<{}, {}, {}, QueryParam.GetAll>;
    type Get = AuthenticatedRequest<{
        id: string;
    }, {}, {}, {
        unflattedResponse: 'true' | 'false';
    }>;
    type Delete = AuthenticatedRequest<{}, {}, IExecutionDeleteFilter>;
    type Retry = AuthenticatedRequest<{
        id: string;
    }, {}, {
        loadWorkflow: boolean;
    }, {}>;
    type Stop = AuthenticatedRequest<{
        id: string;
    }>;
    type GetAllCurrent = AuthenticatedRequest<{}, {}, {}, QueryParam.GetAllCurrent>;
}
export declare namespace MeRequest {
    type UserUpdate = AuthenticatedRequest<{}, {}, UserUpdatePayload>;
    type Password = AuthenticatedRequest<{}, {}, {
        currentPassword: string;
        newPassword: string;
    }>;
    type SurveyAnswers = AuthenticatedRequest<{}, {}, Record<string, string> | {}>;
}
export declare namespace OwnerRequest {
    type Post = AuthenticatedRequest<{}, {}, Partial<{
        email: string;
        password: string;
        firstName: string;
        lastName: string;
    }>, {}>;
}
export declare namespace PasswordResetRequest {
    type Email = AuthlessRequest<{}, {}, Pick<PublicUser, 'email'>>;
    type Credentials = AuthlessRequest<{}, {}, {}, {
        userId?: string;
        token?: string;
    }>;
    type NewPassword = AuthlessRequest<{}, {}, Pick<PublicUser, 'password'> & {
        token?: string;
        userId?: string;
    }>;
}
export declare namespace UserRequest {
    type List = AuthenticatedRequest;
    type Invite = AuthenticatedRequest<{}, {}, Array<{
        email: string;
    }>>;
    type ResolveSignUp = AuthlessRequest<{}, {}, {}, {
        inviterId?: string;
        inviteeId?: string;
    }>;
    type SignUp = AuthenticatedRequest<{
        id: string;
    }, {
        inviterId?: string;
        inviteeId?: string;
    }>;
    type Delete = AuthenticatedRequest<{
        id: string;
        email: string;
        identifier: string;
    }, {}, {}, {
        transferId?: string;
        includeRole: boolean;
    }>;
    type Get = AuthenticatedRequest<{
        id: string;
        email: string;
        identifier: string;
    }, {}, {}, {
        limit?: number;
        offset?: number;
        cursor?: string;
        includeRole?: boolean;
    }>;
    type Reinvite = AuthenticatedRequest<{
        id: string;
    }>;
    type Update = AuthlessRequest<{
        id: string;
    }, {}, {
        inviterId: string;
        firstName: string;
        lastName: string;
        password: string;
    }>;
}
export type LoginRequest = AuthlessRequest<{}, {}, {
    email: string;
    password: string;
}>;
export declare namespace OAuthRequest {
    namespace OAuth1Credential {
        type Auth = AuthenticatedRequest<{}, {}, {}, {
            id: string;
        }>;
        type Callback = AuthenticatedRequest<{}, {}, {}, {
            oauth_verifier: string;
            oauth_token: string;
            cid: string;
        }> & {
            user?: User;
        };
    }
    namespace OAuth2Credential {
        type Auth = OAuth1Credential.Auth;
        type Callback = AuthenticatedRequest<{}, {}, {}, {
            code: string;
            state: string;
        }>;
    }
}
export type NodeParameterOptionsRequest = AuthenticatedRequest<{}, {}, {}, {
    nodeTypeAndVersion: string;
    methodName: string;
    path: string;
    currentNodeParameters: string;
    credentials: string;
}>;
export type NodeListSearchRequest = AuthenticatedRequest<{}, {}, {}, {
    nodeTypeAndVersion: string;
    methodName: string;
    path: string;
    currentNodeParameters: string;
    credentials: string;
    filter?: string;
    paginationToken?: string;
}>;
export declare namespace TagsRequest {
    type GetAll = AuthenticatedRequest<{}, {}, {}, {
        withUsageCount: string;
    }>;
    type Create = AuthenticatedRequest<{}, {}, {
        name: string;
    }>;
    type Update = AuthenticatedRequest<{
        id: string;
    }, {}, {
        name: string;
    }>;
    type Delete = AuthenticatedRequest<{
        id: string;
    }>;
}
export declare namespace NodeRequest {
    type GetAll = AuthenticatedRequest;
    type Post = AuthenticatedRequest<{}, {}, {
        name?: string;
    }>;
    type Delete = AuthenticatedRequest<{}, {}, {}, {
        name: string;
    }>;
    type Update = Post;
}
export declare namespace CurlHelper {
    type ToJson = AuthenticatedRequest<{}, {}, {
        curlCommand?: string;
    }>;
}
export declare namespace LicenseRequest {
    type Activate = AuthenticatedRequest<{}, {}, {
        activationKey: string;
    }, {}>;
}
export type BinaryDataRequest = AuthenticatedRequest<{
    path: string;
}, {}, {}, {
    mode: 'view' | 'download';
    fileName?: string;
    mimeType?: string;
}>;
